{% load wagtailadmin_tags static coderedcms_tags i18n %}
<nav class="nav-main">
    <ul>
        {{ menu_html }}

        <li class="nav-footer" id="footer">
            <div class="account" id="account-settings" title="{% trans 'Edit your account' %}">
                <span class="avatar square avatar-on-dark">
                    <img src="{% avatar_url request.user size=50 %}" alt="" />
                </span>
                <em class="icon icon-arrow-up-after">{{ request.user.first_name|default:request.user.get_username }}</em>
            </div>

            <ul class="nav-footer-submenu">
                <li><a href="{% url 'wagtailadmin_account' %}" class="icon icon-user">{% trans "Account settings" %}</a></li>
                <li><a href="{% url 'wagtailadmin_logout' %}" class="icon icon-logout">{% trans "Log out" %}</a></li>
            </ul>

            <div class="power-by">
              <a href="https://github.com/coderedcorp/coderedcms" target="_blank" title="CodeRed CMS v {% coderedcms_version %}">
                powered by <img src="{% static 'coderedcms/img/codered.png' %}" alt="CodeRed" />
              </a>
            </div>

        </li>
    </ul>
</nav>
