{% extends "coderedcms/pages/web_page_notitle.html" %}
{% load i18n bootstrap4 %}

{% block title %}Password Required{% endblock %}

{% block content %}
<div class="container text-center my-5">
    <div class="row">
        <div class="col-lg-6 offset-lg-3">
            <h1>{% trans "Password Required" %}</h1>
            {% block password_required_message %}
                <p>{% trans "You need a password to access this page." %}</p>
            {% endblock %}
            <form action="{{ action_url }}" method="POST">
                {% csrf_token %}
                {% bootstrap_form form layout='horizontal' %}

                <div class="form-group">
                    <button type="submit" class="btn btn-primary">{% trans 'Continue' %}</button>
                </div>
            </form>
        </div>
    </div>
</div>
{% endblock %}
