# Generated by Django 2.1.7 on 2019-03-22 17:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('coderedcms', '0011_merge_20190322_1309'),
        ('coderedcms', '0010_auto_20190320_1231'),
    ]

    operations = [
    ]
