{% extends "wagtailimportexport/import_from_api.html" %}

{% block css %}
    {{ block.super }}
    {{ form.media.css }}
{% endblock %}

{% block js %}
    {{ block.super }}
    {{ form.media.js }}
{% endblock %}
