{% extends "coderedcms/pages/web_page.html" %}

{% block title %}Page not found{% endblock %}

{% block body_class %}404{% endblock %}

{% block content %}
<div class="container">
    <h1>Page not found</h1>
    <h2>Sorry, this page could not be found.</h2>
</div>
{% endblock %}
