# Generated by Django 2.1.5 on 2019-03-20 02:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('coderedcms', '0009_auto_20190201_1546'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='generalsettings',
            name='robots',
        ),
    ]
