{% load i18n %}

{% if address %}
<div class="my-2">
    <div class="embed-responsive embed-responsive-16by9 mb-5">
        <iframe class="embed-responsive-item" width="100%" style="border:0" src="https://maps.google.com/maps?q={{ address }}&output=embed" title="{% trans 'Map of' %} {{address}}" allowfullscreen></iframe>
    </div>
</div>
{% endif %}
