{% extends "coderedcms/pages/base.html" %}
{% load static %}

{% block custom_assets %}
{# Add your custom CSS here #}
<link rel="stylesheet" type="text/css" href="{% static 'website/css/custom.css' %}">
{% endblock %}

{% block custom_scripts %}
{# Add your custom JavaScript here, or delete the line below if you don't need any #}
<script type="text/javascript" src="{% static 'website/js/custom.js' %}"></script>
{% endblock %}
