/*
CodeRed CMS (https://www.coderedcorp.com/cms/)
Copyright 2018-2021 CodeRed LLC
License: https://github.com/coderedcorp/coderedcms/blob/dev/LICENSE
*/

/* Reset fonts to system */

.title .halloeditor, .title input, .title textarea,
.Draftail-Editor .DraftEditor-root {
    font-family: -apple-system,BlinkMacSystemFont,"Segoe UI",Roboto,sans-serif,"Apple Color Emoji","Segoe UI Emoji","Segoe UI Symbol" !important;
}


/* Inputs and form fields */

.title .halloeditor, .title input, .title textarea {
    font-size:2.5em;
}
.full .halloeditor, .full input, .full textarea {
    padding-top:1em;
    padding-bottom:1em;
    background-color:transparent;
}

.c-sf-block input[type='text'],
.c-sf-block input[type='email'],
.c-sf-block input[type='url'],
.c-sf-block input[type='number'],
.c-sf-block input[type='date'],
.c-sf-block input[type='time'],
.c-sf-block input[type='datetime'],
.c-sf-block input[type='datetime-local'],
.c-sf-block textarea,
.c-sf-block select,
.c-sf-block .richtext,
.c-sf-block .tagit {
    background-color:white;
    border:1.5px solid #ddd;
}

.c-sf-block input:focus,
.c-sf-block textarea:focus,
.c-sf-block select:focus,
.c-sf-block .richtext:focus,
.c-sf-block .tagit:focus {
    border-color:#00b0b1;
    background-color:#f2fcfc;
}

.c-sf-button {
    max-width: 200px;
}

.input input,
.input select {
    width: 300px;
    max-width: 100%;
}
.input input[type='radio'],
.input input[type='checkbox'] {
    width: auto;
}
.input textarea,
.full .input input {
    width:100%;
}
.input input[type='color']{
    height: 40px;
    padding: 5px;
}


/* Override and enhance the streamfield editor to support our deeply nested streamfields. */

.chooser .unchosen::before, .chooser .chosen::before {
    color:rgba(0,0,0,0.2)
}

.c-sf-block {
    display: block;
}

.c-sf-block:hover,
.c-sf-block:focus {
    box-shadow: 0 0 20px rgba(0,0,0,0.2) !important;
}

.c-sf-add-panel,
.c-sf-container {
    padding: 0 !important;
    width:100%;
    flex-basis: 100%;
}

.field.char_field.widget-draftail_rich_text_area .field-content,
.field.char_field.widget-textarea .field-content {
    width:100%;
    max-width:100%;
}

.sequence-controls {
    border: none;
    background-color: transparent;
    top: unset;
}
.sequence-controls h3 label {
    font-weight: bold;
}

.tab-content {
    width: 100%;
}

.object.stream-field {
    background-color: inherit;;
}

.object > h2,
.object > .title-wrapper label {
    font-weight: 600;
}

.object .multiple {
    width:100%;
    max-width:100%;
}

.object fieldset {
    width:100%;
    max-width:100%;
}
.object > fieldset {
    padding-top:6em;
}
.object > fieldset fieldset {
    padding-top:0;
}

.stream-menu.stream-menu-closed .toggle {
    color:#888;
}

.fields > li, .field-col {
    padding-bottom:0.5em;
}

.tagit, input, select, textarea {
    color:#000;
}
.tagit, input, select, textarea {
    font-weight:normal;
}

.stream-menu-inner h3 {
    color: #ccc;
    text-transform: uppercase;
    font-weight: 700;
    border-bottom: 2px solid rgba(255,255,255,0.1);
    padding: 0.5em;
}


/* Draftail */

.Draftail-Toolbar {
    border: none !important;
    border-radius:0 !important;
    color:rgba(0,0,0,0.2) !important;
    padding: 0 !important;
    transition: all ease .25s;
}
.Draftail-Toolbar:hover {
    color:rgba(0,0,0,0.7) !important;
    transition: all ease .25s;
}
.Draftail-Toolbar .Draftail-ToolbarButton--active {
    background-color:rgba(0,0,0,0.05) !important;
    border:none;
    transition: all ease .25s;
}
.Draftail-Toolbar:hover .Draftail-ToolbarButton--active {
    background-color:rgba(0,0,0,0.2) !important;
    transition: all ease .25s;
}
.Draftail-Toolbar .Draftail-ToolbarGroup + .Draftail-ToolbarGroup::before {
    content: none;
}
.Draftail-Toolbar .Draftail-ToolbarGroup + .Draftail-ToolbarGroup {
    margin-left: 1em;
}

.Draftail-Editor .DraftEditor-root {
    color:#000 !important;
}
.Draftail-DividerBlock {
    background-color:#ccc !important;
}


/* Custom */

.codered-collapsible {
    padding:0;
}

.codered-collapsible .codered-collapsible-target {
    padding: 10px 0 0;
}

.codered-checkbox-group {
    margin-bottom: 1em;
}
.codered-checkbox-group ul {
    margin-top: 0.5em;
}
.codered-checkbox-group label {
    padding-bottom: 0.25em;
}
.codered-checkbox-group label input {
    margin-right: 0.25em;
}

.codered-callout {
    background-color: #f0f1f2;
    border-radius: 6px;
    display: inline-block;
    padding: 2em 3em;
    text-align: center;
}
.codered-callout .codered-big-icon {
    font-size: 10rem;
    color: rgba(0,0,0,0.1);
}
.codered-callout h3 {
    font-size: 1.2rem;
    font-weight: 600;
}
.codered-callout p {
    font-size: 1rem;
}
.codered-callout .codered-new {
    border-radius: 4px;
    background-color: rgba(0,0,0,0.1);
    font-size: 0.6em;
    padding: 0.3em 0.6em;
    text-transform: uppercase;
}
