<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8" />
        <meta http-equiv="X-UA-Compatible" content="IE=edge" />
        <title>Internal server error</title>
        <meta name="viewport" content="width=device-width, initial-scale=1" />
        <style>
        body {
            font-family: -apple-system,BlinkMacSystemFont,"Segoe UI",Roboto,"Helvetica Neue",Arial,sans-serif,"Apple Color Emoji","Segoe UI Emoji","Segoe UI Symbol";
            line-height:1.5;
            padding:2em;
        }
        hr { border: 0.5px solid #ddd; }
        h1, h2 { font-weight:normal; }
        </style>
    </head>
    <body>
        <h1>Internal server error</h1>
        <hr>
        <h2>Sorry, our site has experienced an error.</h2>
        <p>
            The error has been logged and reported to an engineer.
            Please try again soon. If the issue persists, contact us to resolve.
        </p>
    </body>
</html>
