{% extends "wagtailadmin/base.html" %}
{% load wagtailimages_tags wagtailsettings_tags %}


{% block branding_logo %}
    {% if settings.coderedcms.LayoutSettings.logo %}
        {% image settings.coderedcms.LayoutSettings.logo max-300x300 as logo_image %}
        <img src="{{ logo_image.url }}" class="logo-custom" alt="Dashboard"/>
    {% else %}
        {{block.super}}
    {% endif %}
{% endblock %}
