# Generated by Django 2.1.7 on 2019-03-22 17:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('coderedcms', '0010_remove_generalsettings_robots'),
        ('coderedcms', '0010_generalsettings_email'),
    ]

    operations = [
    ]
